﻿using System;
using System.Diagnostics;
using System.Windows.Input;

namespace CommandSample
{
    /// <summary>
    /// RelayCommand is a class written by Josh Smith as documented in MSDN Magazine, 2009 February Issue.
    /// URL: http://msdn.microsoft.com/en-us/magazine/dd419663.aspx
    /// It is a comprhensive solution for creating bindalbe command quickly.
    /// The Prism DelgateCommand is very similar.
    /// </summary>
    public class RelayCommand : ICommand
    {
        #region Fields

        readonly Action<object> _execute;
        readonly Predicate<object> _canExecute;

        #endregion // Fields

        #region Constructors

        public RelayCommand(Action<object> execute)
            : this(execute, null)
        {
        }

        public RelayCommand(Action<object> execute, Predicate<object> canExecute)
        {
            if (execute == null)
                throw new ArgumentNullException("execute");

            _execute = execute;
            _canExecute = canExecute;
        }
        #endregion // Constructors

        #region ICommand Members

        [DebuggerStepThrough]
        public bool CanExecute(object parameter)
        {
            return _canExecute == null ? true : _canExecute(parameter);
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public void Execute(object parameter)
        {
            _execute(parameter);
        }

        #endregion // ICommand Members
    }

    /// <summary>
    /// A command whose sole purpose is to relay its functionality to other objects by invoking delegates. The default return value for the CanExecute method is 'true'.
    /// </summary>
    /// <typeparam name="T">Type of parameter</typeparam>
    public class RelayCommand<T> : ICommand
    {
        #region Declarations

        /// <summary>
        /// Indicates whether the command can be executed
        /// </summary>
        private readonly Predicate<T> canExecute;

        /// <summary>
        /// Action Predicate
        /// </summary>
        private readonly Action<T> execute;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class and the command can always be executed.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        public RelayCommand(Action<T> execute)
            : this(execute, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        /// <param name="canExecute">The execution status logic.</param>
        public RelayCommand(Action<T> execute, Predicate<T> canExecute)
        {
            if (execute == null)
            {
                throw new ArgumentNullException("execute");
            }

            this.execute = execute;
            this.canExecute = canExecute;
        }

        #endregion

        #region ICommand Members

        /// <summary>
        /// Wires up the CommandManager's RequerySuggested event
        /// </summary>
        public event EventHandler CanExecuteChanged
        {
            add
            {
                if (this.canExecute != null)
                {
                    CommandManager.RequerySuggested += value;
                }
            }

            remove
            {
                if (this.canExecute != null)
                {
                    CommandManager.RequerySuggested -= value;
                }
            }
        }

        /// <summary>
        /// Indicates whether the command is in an executable state
        /// </summary>
        /// <param name="parameter">Command specific parameter</param>
        /// <returns>A boolean indicating whether the command is in an executable state</returns>        
        public bool CanExecute(object parameter)
        {
            return this.canExecute == null ? true : parameter == null ? false : this.canExecute((T)parameter);
        }

        /// <summary>
        /// Executes the command
        /// </summary>
        /// <param name="parameter">Command specific parameter</param>
        public void Execute(object parameter)
        {
            this.execute((T)parameter);
        }

        #endregion
    }
}
